package com.jjt.check.service.impl;

import com.jjt.check.domain.CheckRecord;
import com.jjt.check.domain.CheckRecordDetail;
import com.jjt.check.mapper.CheckRecordDetailMapper;
import com.jjt.check.mapper.CheckRecordMapper;
import com.jjt.check.service.ICheckRecordService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 自动巡检记录Service业务层处理
 *
 * @author jjt
 * @date 2024-12-06
 */
@Service
public class CheckRecordServiceImpl implements ICheckRecordService {
    @Resource
    private CheckRecordMapper checkRecordMapper;
    @Resource
    private CheckRecordDetailMapper crdMapper;

    /**
     * 查询自动巡检记录
     *
     * @param crId 自动巡检记录主键
     * @return 自动巡检记录
     */
    @Override
    public CheckRecord selectCheckRecordByCrId(Long crId) {
        CheckRecord cr;
        if (crId == 0L) {
            cr = checkRecordMapper.selectCheckRecordByLast();
        } else {
            cr = checkRecordMapper.selectCheckRecordByCrId(crId);
        }
        CheckRecordDetail crd = new CheckRecordDetail();
        crd.setCrId(cr.getCrId());
        List<CheckRecordDetail> list = crdMapper.selectCheckRecordDetailList(crd);
        cr.setDetailList(list);
        return cr;
    }

    /**
     * 查询自动巡检记录列表
     *
     * @param checkRecord 自动巡检记录
     * @return 自动巡检记录
     */
    @Override
    public List<CheckRecord> selectCheckRecordList(CheckRecord checkRecord) {
        return checkRecordMapper.selectCheckRecordList(checkRecord);
    }

    /**
     * 新增自动巡检记录
     *
     * @param checkRecord 自动巡检记录
     * @return 结果
     */
    @Override
    public int insertCheckRecord(CheckRecord checkRecord) {
        return checkRecordMapper.insertCheckRecord(checkRecord);
    }

    /**
     * 修改自动巡检记录
     *
     * @param checkRecord 自动巡检记录
     * @return 结果
     */
    @Override
    public int updateCheckRecord(CheckRecord checkRecord) {
        return checkRecordMapper.updateCheckRecord(checkRecord);
    }

    /**
     * 批量删除自动巡检记录
     *
     * @param crIds 需要删除的自动巡检记录主键
     * @return 结果
     */
    @Override
    public int deleteCheckRecordByCrIds(Long[] crIds) {
        return checkRecordMapper.deleteCheckRecordByCrIds(crIds);
    }

    /**
     * 删除自动巡检记录信息
     *
     * @param crId 自动巡检记录主键
     * @return 结果
     */
    @Override
    public int deleteCheckRecordByCrId(Long crId) {
        return checkRecordMapper.deleteCheckRecordByCrId(crId);
    }
}
